[![NPM version][npm-image]][npm-url]
[![build status][travis-image]][travis-url]
[![Test coverage][coveralls-image]][coveralls-url]
[![David deps][david-image]][david-url]
[![node version][node-image]][node-url]
[![npm download][download-image]][download-url]
[![npm license][license-image]][download-url]

[npm-image]: https://img.shields.io/npm/v/lei-stream.svg?style=flat-square
[npm-url]: https://npmjs.org/package/lei-stream
[travis-image]: https://img.shields.io/travis/leizongmin/node-lei-stream.svg?style=flat-square
[travis-url]: https://travis-ci.org/leizongmin/node-lei-stream
[coveralls-image]: https://img.shields.io/coveralls/leizongmin/node-lei-stream.svg?style=flat-square
[coveralls-url]: https://coveralls.io/r/leizongmin/node-lei-stream?branch=master
[david-image]: https://img.shields.io/david/leizongmin/node-lei-stream.svg?style=flat-square
[david-url]: https://david-dm.org/leizongmin/node-lei-stream
[node-image]: https://img.shields.io/badge/node.js-%3E=_4.0-green.svg?style=flat-square
[node-url]: http://nodejs.org/download/
[download-image]: https://img.shields.io/npm/dm/lei-stream.svg?style=flat-square
[download-url]: https://npmjs.org/package/lei-stream
[license-image]: https://img.shields.io/npm/l/lei-stream.svg

# lei-stream
Read/Write Stream line by line 按行读写流
## 安装

```bash
$ npm install lei-stream --save
```

**需要 Node.js v4.0.0 或更改版本**


## readLine 按行读取流

使用方法1：

```javascript
'use strict';

const readLine = require('lei-stream').readLine;

readLine('./myfile.txt').go((data, next) => {
  console.log(data);
  next();
}, function () {
  console.log('end');
});
```

使用方法2：

```javascript
'use strict';

const fs = require('fs');
const readLine = require('lei-stream').readLine;

// readLineStream第一个参数为ReadStream实例，也可以为文件名
const s = readLine(fs.createReadStream('./myfile.txt'), {
  // 换行符，默认\n
  newline: '\n',
  // 是否自动读取下一行，默认false
  autoNext: false,
  // 编码器，可以为函数或字符串（内置编码器：json，base64），默认null
  encoding: function (data) {
    return JSON.parse(data);
  }
});

// 读取到一行数据时触发data事件
s.on('data', (data) => {
  console.log(data);
  s.next();
});

// 流结束时触发end事件
s.on('end', () => {
  console.log('end');
});

// 读取时出错
s.on('error', (err) => {
  console.error(err);
});
```

## writeLine

按行写流

```javascript
'use strict';

const fs = require('fs');
const writeLineStream = require('lei-stream').writeLine;

// writeLineStream第一个参数为WriteStream实例，也可以为文件名
const s = writeLineStream(fs.createWriteStream('./myfile.txt'), {
  // 换行符，默认\n
  newline: '\n',
  // 编码器，可以为函数或字符串（内置编码器：json，base64），默认null
  encoding: function (data) {
    return JSON.stringify(data);
  },
  // 缓存的行数，默认为0（表示不缓存），此选项主要用于优化写文件性能，写入的内容会先存储到缓存中，当内容超过指定数量时再一次性写入到流中，可以提高写速度
  cacheLines: 0
});

// 写一行
s.write(data, () => {
  // 回调函数可选
  console.log('wrote');
});

// 结束
s.end(() => {
  // 回调函数可选
  console.log('end');
});

// 写时出错
s.on('error', (err) => {
  console.error(err);
});
```


## TailStream

监听文件新增内容变化的流（返回的是一个`Readable Stream`,具体可参考 Node.js API 文档）

```javascript
'use strict';

const tailStream = require('lei-stream').tailStream;

// 创建流
const s = tailStream('./myfile.txt', {
  position: 'end', // end表示监听之前先定位到文件末尾,否则会先读取出文件之前的所有内容再开始监听
});

// 有新内容会触发data事件
s.on('data', data => {
  console.log(data);
});
````



## License

```
The MIT License (MIT)

Copyright (c) 2015-2017 Zongmin Lei <leizongmin@gmail.com>
http://ucdok.com

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
```
